/**
 * Js for the Advanced Sidebar Menu widgets
 *
 * @author Mat Lipe
 *
 */
/* global asm_pro_config */
var asm_pro = window.asm_pro || {};
(function ($, a, config) {

	/**
	 * Accordion fields
	 */
	a.accordion = {
		init: function () {
			$('[data-js="advanced-sidebar-menu-pro/widget/accordion-style"]').change(this._change_style_display);
		},

		/**
		 * Change the icons displayed in the "Accordion icons style" field
		 * within the widget.
		 *
		 * @private
		 */
		_change_style_display: function () {
			var el = $(this);
			var styles = config.accordion_styles[el.val()];
			$('[data-select="' + el.attr('id') + '"]').html(styles.expand + ' ' + styles.contract);
		},
	};


	/**
	 * Style fields
	 */
	a.styles = {

		init: function () {
			this._handle_style_fields();
			this._init_color_pickers();
		},


		/**
		 * So the widget may be saved when all we do is change
		 * a color from a picker.
		 *
		 * @private
		 */
		_init_color_pickers() {
			var select = $(a.widget_id + ' .asm-color-select');
			select.wpColorPicker({
				change: _.throttle(function () { // For Customizer
					select.parents().find('.widget-inside').trigger('change');
				}, 3000)
			});
		},

		/**
		 * Show/Display the style fields.
		 * Change the button text depending on state.
		 *
		 * @private
		 */
		_handle_style_fields: function () {
			$(a.widget_id + ' [data-js="advanced-sidebar-menu-pro/widget/show-styles"]').click( function () {
				var el = $(this);
				var target = $('#' + el.data('target'));
				target.slideToggle(function () {
					if (target.is(':visible')) {
						el.text(config.i18n.hide_styles);
					} else {
						el.text(config.i18n.show_styles);
					}
				});
			});
		},
	};

	a.widget_id = null;

	/**
	 * Triggered via inline js to account for the form
	 * updates via ajax as WP saves and adds widgets
	 */
	a.init = function ($widget_id) {
		a.widget_id = $widget_id;
		a.styles.init();
		a.accordion.init();
	};


})(jQuery, asm_pro, asm_pro_config);
