/**
 * Js for the Advanced Sidebar Menu Pro widgets (front-end)
 *
 * @author Mat Lipe
 *
 * @since 1.5
 *
 */
var asm_pro = window.asm_pro || {};
(function( $, a, config ){
	a.accordion = {
		_enabled_accordions : {},

		init : function(){
			this._render_accordions();
		},

		/**
		 * Used by the widget output to register an accordion
		 * menu.
		 * If the widget has accordions enabled it will end up here
		 *
		 * @param {string} $widget_id
		 * @param {string} $instance - widget instance
		 */
		register_accordion : function( $widget_id, $instance ){
			this._enabled_accordions[$widget_id] = $instance;
		},

		/**
		 * Iterate through the registered accordions
		 * sets up the icons
		 * hides the child menus
		 *
		 * @uses this._enabled_accordions
		 *
		 * @private
		 */
		_render_accordions : function(){
			var self = a.accordion;
			var parents = [];

			for( var $widget_id in this._enabled_accordions ){
				if( !this._enabled_accordions.hasOwnProperty( $widget_id ) ){
					continue;
				}

				var el = $( '[data-js="' + $widget_id + '"]' );
				//the .has() only target elements that have actual children in
				//case of 'always display child pages' not being active
				if( this._enabled_accordions[ $widget_id ][ 'include_parent' ] && !this._enabled_accordions[ $widget_id ][ 'include_parent_in_accordion' ]){
					parents = el.find( '> ul > li .has_children' ).has( 'ul' );
				} else {
					parents = el.find( 'ul .has_children' ).has( 'ul' );
				}


				parents.each( function(){
					var _parent_menu = $( this );
					var _icon = {};
					//current page and parents of current page should
					//be open
					if( !a.accordion._enabled_accordions[ $widget_id ][ 'keep_closed' ]
						&& (
						_parent_menu.hasClass( 'current_page_item' )
						|| _parent_menu.hasClass( 'current-cat' )
						|| _parent_menu.hasClass( 'current-menu-item' )
						|| _parent_menu.find( '.current-cat' ).length
						|| _parent_menu.find( '.current_page_item' ).length
						|| _parent_menu.find( '.current-menu-item' ).length
						)
					){
						_parent_menu.addClass( 'open' );
						_icon = self._get_icon( $widget_id, 'contract', _parent_menu );
						//everything else should be closed
					} else {
						_parent_menu.addClass( 'closed' );
						_parent_menu.find( '> ul' ).hide();
						_icon = self._get_icon( $widget_id, 'expand', _parent_menu );
					}

					_parent_menu.find( '> a' ).after( _icon );

				} );

			}
		},

		/**
		 * Icon was clicked. We are either going to expand
		 * or contract a menu based on the icon's state.
		 * After the menu is changed the icon will change
		 * to the new one here.
		 *
		 * @param {jQuery} parent_menu - menu we are working with
		 * @param {jQuery} icon - icon which was clicked
		 * @private
		 */
		_expand_sub_menu : function( parent_menu, icon ){
			if( icon.data( 'state' ) == 'expand' ){
				parent_menu
					.addClass( 'open' )
					.removeClass( 'closed' );

				parent_menu.find( '> ul' ).slideDown();
				_icon = this._get_icon( icon.data( 'widget_id' ), 'contract', parent_menu );
				icon.replaceWith( _icon );
			} else {
				parent_menu
					.addClass( 'closed' )
					.removeClass( 'open' );

				parent_menu.find( '> ul' ).slideUp();
				_icon = this._get_icon( icon.data( 'widget_id' ), 'expand', parent_menu );
				icon.replaceWith( _icon );
			}
		},


		/**
		 *
		 * @param {string} $widget_id
		 * @param {string} $state - expand|contract
		 * @param {jQuery} parent_menu - the menu we are working with
		 *
		 * @returns {jQuery}
		 * @private
		 */
		_get_icon : function( $widget_id, $state, parent_menu ){
			var $style = this._enabled_accordions[ $widget_id ][ config.accordion_style_key ];
			var icon = $( config.accordion_styles[ $style ][ $state ] );
			icon.data( 'state', $state );
			icon.data( 'widget_id', $widget_id );

			icon.click( this._expand_sub_menu.bind( this, parent_menu, icon ) );
			return icon;
		}

	};

	$( function(){
		a.accordion.init();
		//event on accordions loaded @since 3.4.1
		$( document ).trigger( 'advanced-sidebar-menu-pro/accordions/loaded', a.accordion );
	} );

	//event to register accordions @since 3.4.4
	$( document ).trigger( 'advanced-sidebar-menu-pro/accordions/available', a.accordion );

})( jQuery, asm_pro, asm_pro_config );
