<?php

/**
 * Advanced_Sidebar_Menu_Pro__Menus__Navigation
 *
 * @author Mat Lipe
 * @since  3.3.0
 *
 */
class Advanced_Sidebar_Menu_Pro__Menus__Navigation extends Advanced_Sidebar_Menu_Menus_Abstract {
	const HOOK_PRIORITY = 10;

	protected $tree = array();

	protected $menu_items = array();


	/**
	 * @see  Advanced_Sidebar_Menu_Pro__Menus__Navigation::remove_classes_hooks()
	 *
	 * @todo find a more appropriate place to put all classes hooks
	 *
	 * @return void
	 */
	public function add_classes_hooks() {
		//@notice this requires WP 4.8.0+
		add_filter( 'nav_menu_submenu_css_class', array(
			$this,
			'add_child_sidebar_menu_class',
		), self::HOOK_PRIORITY, 3 );

		add_filter( 'wp_nav_menu_objects', array( $this, 'add_has_children_class' ), self::HOOK_PRIORITY );
		add_filter( 'wp_nav_menu_objects', array( $this, 'exclude_non_family_items' ), self::HOOK_PRIORITY );

	}


	/**
	 * @see  Advanced_Sidebar_Menu_Pro__Menus__Navigation::add_classes_hooks()
	 *
	 * @todo find a more appropriate place to put all classes hooks
	 *
	 * @return void
	 */
	protected function remove_classes_hooks() {
		remove_filter( 'nav_menu_submenu_css_class', array(
			$this,
			'add_child_sidebar_menu_class',
		), self::HOOK_PRIORITY );
		remove_filter( 'wp_nav_menu_objects', array( $this, 'add_has_children_class' ), self::HOOK_PRIORITY );
		remove_filter( 'wp_nav_menu_objects', array( $this, 'exclude_non_family_items' ), self::HOOK_PRIORITY );
	}


	/**
	 * Is this widget being displayed
	 *
	 * @return bool
	 */
	public function is_displayed() {
		if ( empty( $this->instance[ Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::NAVIGATION_MENU ] ) ) {
			return false;
		}
		$nav_menu = wp_get_nav_menu_object( $this->instance[ Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::NAVIGATION_MENU ] );
		if ( empty( $nav_menu ) ) {
			return false;
		}

		if ( $this->checked( Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::CURRENT_PARENT_ONLY ) ) {
			if ( $this->checked( Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::DISPLAY_NO_CURRENT ) ) {
				return true;
			}
			$menu_items = wp_get_nav_menu_items( $nav_menu );
			_wp_menu_item_classes_by_context( $menu_items );
			$objects = Advanced_Sidebar_Menu_Pro__Nav_Menu__Nav_Menu_Objects::factory( $menu_items );

			return (bool) $objects->get_current_item();
		}

		return true;
	}


	/**
	 * Exclude any items not part of the current menu item's family, if settings say to do so
	 *
	 * @since 3.3.1
	 *
	 * @param array $menu_items
	 *
	 * @return array
	 */
	public function exclude_non_family_items( array $menu_items ) {
		if ( ! $this->checked( Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::CURRENT_PARENT_ONLY ) ) {
			return $menu_items;
		}

		$objects = Advanced_Sidebar_Menu_Pro__Nav_Menu__Nav_Menu_Objects::factory( $menu_items );
		$current = $objects->get_current_item();

		//Not viewing a menu item and want the top level to display
		if ( false === $current && $this->checked( Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::DISPLAY_NO_CURRENT ) ) {
			$objects->remove_all_child_items();

			return $objects->flatten();
		}


		if ( false !== $current && $this->checked( Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::TOP_LEVEL_SIBLINGS ) && false === $current->get_parent() ) {
			$count = count( $objects->get_current_item_family() );
			//no children in family so we display the top level items
			if ( $count < 2 ) {
				$objects->remove_all_child_items();
			} else {
				//there are children available so we display the family only
				$objects->remove_all_non_family_items();
			}

			return $objects->flatten();
		}

		//default action when option is checked
		$objects->remove_all_non_family_items();

		return $objects->flatten();

	}


	/**
	 * @filter wp_nav_menu_objects 10 2
	 *
	 * @since  3.3.1
	 *
	 * @param array $menu_items
	 *
	 * @return array
	 */
	public function add_has_children_class( array $menu_items ) {
		foreach ( $menu_items as $_item ) {
			if ( in_array( 'menu-item-has-children', $_item->classes, true ) ) {
				$_item->classes[] = 'has_children';
			}
			$_item->classes = array_unique( $_item->classes );
		}

		return $menu_items;
	}


	/**
	 * @filter nav_menu_submenu_css_class 10 3
	 *
	 * @param array $classes
	 *
	 * @return array
	 */
	public function add_child_sidebar_menu_class( array $classes, $args, $depth ) {
		if ( 0 === $depth ) {
			return array( 'child-sidebar-menu' );
		}

		return array( 'children' );
	}


	/**
	 * Get the arguments to be sent to wp_nav_menu
	 *
	 * @return array
	 */
	public function get_nav_menu_args() {
		$_args = array(
			'fallback_cb' => '',
			'menu'        => wp_get_nav_menu_object( $this->instance[ Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::NAVIGATION_MENU ] ),
			'echo'        => false,
			'container'   => false,
			'menu_class'  => 'parent-sidebar-menu',
		);

		return apply_filters( 'advanced-sidebar-menu/menus/navigation/get-nav-menu-args', $_args, $this );

	}


	public function render() {
		if ( ! $this->is_displayed() ) {
			return;
		}

		$this->add_classes_hooks();
		//to allow block styles to work
		$this->instance[ Advanced_Sidebar_Menu_Menus_Abstract::INCLUDE_PARENT ] = true;

		// phpcs:disable
		echo $this->args['before_widget'];

		do_action( 'advanced-sidebar-menu/menus/navigation/render', $this );

		if ( $this->checked( self::USE_PLUGIN_STYLES ) ) {
			Advanced_Sidebar_Menu_Core::instance()->include_plugin_styles();
		}

		if ( ! empty( $this->instance[ Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::TITLE ] ) ) {
			echo $this->args['before_title'] . $this->instance[ Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu::TITLE ] . $this->args['after_title'];
		}

		$output = wp_nav_menu( $this->get_nav_menu_args() );

		echo apply_filters( 'advanced-sidebar-menu/widget/navigation-menu/output', $output, $this->args, $this->instance, $this );

		echo $this->args['after_widget'];
		// phpcs:enable

		$this->remove_classes_hooks();
	}


	public function get_top_parent_id() {
		//CURRENTLY NOT APPLICABLE
	}


	public function get_order_by() {
		//CURRENTLY NOT APPLICABLE
	}


	public function get_order() {
		//CURRENTLY NOT APPLICABLE
	}


	public function get_levels_to_display() {
		//CURRENTLY NOT APPLICABLE
	}


	public function get_menu_depth() {
		//CURRENTLY NOT APPLICABLE
	}


	/**************** static *****************/
	/**
	 * Mostly here to call the parent _factory() in a PHP 5.2 structure
	 *
	 * @param array $widget_instance
	 * @param array $widget_args
	 *
	 * @static
	 *
	 * @return \Advanced_Sidebar_Menu_Pro__Menus__Navigation
	 */
	public static function factory(
		array $widget_instance, array $widget_args
	) {
		return parent::_factory( __CLASS__, $widget_instance, $widget_args );
	}


}
