<?php

/**
 * Nav Menu Objects
 *
 * @author Mat Lipe
 * @since  3.3.3
 *
 */
class Advanced_Sidebar_Menu_Pro__Nav_Menu__Nav_Menu_Objects {
	/**
	 * menu_items
	 *
	 * @var Advanced_Sidebar_Menu_Pro__Nav_Menu__Item[]
	 */
	protected $menu_items;

	protected $hierarchy = array();

	protected $current_family;


	public function get_item( $post_id ) {
		if ( ! isset( $this->menu_items[ $post_id ] ) ) {
			return false;
		}

		return $this->menu_items[ $post_id ];
	}


	/**
	 * If we only want the top level items
	 *
	 * @return void
	 */
	public function remove_all_child_items() {
		foreach ( $this->menu_items as $_id => $_item ) {
			if ( 0 !== (int) $_item->menu_item_parent ) {
				unset( $this->menu_items[ $_id ] );
			}
		}
	}


	/**
	 * If we only want items which are part of
	 * the current item's hierarchy
	 *
	 * @return void
	 */
	public function remove_all_non_family_items() {
		foreach ( $this->menu_items as $_id => $_item ) {
			if ( ! $_item->is_in_current_family() ) {
				unset( $this->menu_items[ $_id ] );
			}
		}
	}


	public function is_in_current_family( $post_id ) {
		$family = $this->get_current_item_family();

		return in_array( (int) $post_id, $family, true );
	}


	/**
	 * Get all ids which are parent, children, current of the
	 * menu item
	 *
	 * @return int[]
	 */
	public function get_current_item_family() {
		if ( null !== $this->current_family ) {
			return $this->current_family;
		}

		$top = $this->get_current_top_level_parent();
		if ( empty( $top ) ) {
			return array();
		}

		$children = $this->get_child_items_recursive( $top->get_id() );

		return $this->current_family = array_map( array( $this, 'flatten_to_id' ), $children );

	}


	/**
	 *
	 * @param $post_id
	 *
	 * @return Advanced_Sidebar_Menu_Pro__Nav_Menu__Item[]
	 */
	public function get_child_items( $post_id ) {
		$items = array();
		if ( ! empty( $this->hierarchy[ $post_id ] ) ) {
			foreach ( $this->hierarchy[ $post_id ] as $_child ) {
				$items[ $_child ] = $this->menu_items[ $_child ];
			}
		}

		return $items;
	}


	public function get_current_top_level_parent() {
		$current = $this->get_current_item();
		if ( ! empty( $current ) ) {
			while ( false !== $current->get_parent() ) {
				$current = $current->get_parent();
			}
		}

		return $current;
	}


	public function get_current_item() {
		$current = wp_filter_object_list( $this->menu_items, array( 'current' => true ) );
		$post    = reset( $current );

		return empty( $post ) ? false : $post;
	}


	/**
	 *
	 *
	 * @return WP_Post[]
	 */
	public function flatten() {
		$menu_items = array();
		foreach ( $this->menu_items as $_item ) {
			if ( ! $_item->is_excluded() ) {
				$menu_items[] = $_item->get();
			}
		}

		return $menu_items;
	}


	/**
	 * @internal
	 *
	 * @return array
	 */
	public function get_hierarchy() {
		return $this->hierarchy;
	}


	/**
	 *
	 * @param $item Advanced_Sidebar_Menu_Pro__Nav_Menu__Item
	 *
	 * @return int
	 */
	protected function flatten_to_id( $item ) {
		return $item->get_id();
	}


	/**
	 * Get all the child, grandchild, self, great grandchild etc.
	 *
	 * @param $post_id
	 *
	 * @return mixed
	 */
	protected function get_child_items_recursive( $post_id ) {
		$items = array( array( $this->menu_items[ $post_id ] ) );
		if ( ! empty( $this->hierarchy[ $post_id ] ) ) {
			foreach ( $this->hierarchy[ $post_id ] as $_child_id ) {
				$items[] = $this->get_child_items_recursive( $_child_id );
			}
		}

		return call_user_func_array( 'array_merge', $items );
	}


	protected function parse_hierarchy() {
		foreach ( $this->menu_items as $_item ) {
			$this->hierarchy[ (int) $_item->menu_item_parent ][] = $_item->get_id();
		}
	}


	/**
	 *
	 * @param WP_Post[] $menu_items
	 *
	 * @static
	 *
	 * @return Advanced_Sidebar_Menu_Pro__Nav_Menu__Nav_Menu_Objects
	 */
	public static function factory( array $menu_items ) {
		$objects = new self();
		foreach ( $menu_items as $_item ) {
			$objects->menu_items[ (int) $_item->ID ] = Advanced_Sidebar_Menu_Pro__Nav_Menu__Item::factory( $_item, $objects );
		}
		$objects->parse_hierarchy();

		return $objects;
	}

}
