<?php

/**
 * Styles
 *
 * @author Mat Lipe
 */
class Advanced_Sidebar_Menu_Pro_Styles {
	const BLOCK_STYLE = 'block_style';
	const BORDER = 'border';
	const BORDER_COLOR = 'border_color';
	/**
	 * @todo break this out into it own setting and support things
	 *       like bullet colors or custom bullets.
	 */
	const BULLET_STYLE = 'bullet_style';


	public function hook() {
		add_action( 'advanced-sidebar-menu/widget/page/after-form', array( $this, 'widget_scripts' ) );

		// UGH! beaver builder hack
		if ( isset( $_GET['fl_builder'] ) ) {
			add_action( 'wp_enqueue_scripts', array( $this, 'load_scripts_on_front_end' ) );
		}
	}


	/**
	 *
	 * @param array                                 $instance
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget
	 * @param array                                 $fields
	 *
	 * @since 3.2.0
	 *
	 * @return mixed|void
	 */
	public function render_fields( array $instance, $widget, array $fields ) {
		?>
		<div class="advanced-sidebar-menu-full-width" style="width:100%; display:none;" data-advanced-sidebar-menu-hide="0">
			<div class="advanced-sidebar-menu-column-box" style="text-align: center;">
				<div
					class="button-secondary widefat"
					data-js="advanced-sidebar-menu-pro/widget/show-styles"
					data-target="<?php echo esc_js( $widget->id ); ?>-css">
					<?php esc_html_e( 'Click to show styles', 'advanced-sidebar-menu' ); ?>
				</div>
			</div>
		</div>

		<div id="<?php echo esc_attr( $widget->id ); ?>-css" data-advanced-sidebar-menu-hide="1" class="asm-css-styles">
			<div class="advanced-sidebar-menu-column">
				<div class="advanced-sidebar-menu-column-box">
					<?php
					$count = 1;
					foreach ( $fields

					as $field => $_label ) {
						$count ++;
						if ( 16 === $count ) {
							?>
				</div>
			</div>
			<div class="advanced-sidebar-menu-column advanced-sidebar-menu-column-right">
				<div class="advanced-sidebar-menu-column-box">
							<?php
						}
						if ( 'divider' === $_label ) {
							?>
				</div>
				<div class="advanced-sidebar-menu-column-box">
					<p>
							<?php
							continue;
						}
						?>
						<label>
							<?php echo esc_html( $_label ); ?>:
						</label>

						<?php
						// font weights
						if ( strpos( $field, 'weight' ) !== false ) {
							printf( '<select name="%s" id="%s">', esc_attr( $widget->get_field_name( $field ) ), esc_attr( $widget->get_field_id( $field ) ) );
							?>
							<option value="">
								<?php esc_html_e( '- default -', 'advanced-sidebar-menu' ); ?>
							</option>
							<?php
							foreach ( (array) self::get_instance()->get_font_weights() as $weight => $__label ) {
								printf( '<option value="%s" %s>%s</option>', esc_attr( $weight ), selected( $instance[ $field ], $weight ), esc_html( $__label ) );
							}
							?>
							</select>
							<?php
							// colors
						} elseif ( strpos( $field, 'color' ) !== false ) {
							printf( '<input type="text" name="%s" value="%s" id="%s" class="asm-color-select">', esc_attr( $widget->get_field_name( $field ) ), esc_html( $instance[ $field ] ), esc_attr( $widget->get_field_id( $field ) ) );

							// size
						} elseif ( strpos( $field, 'size' ) !== false ) {
							printf( '<input type="number" step="1" min="2" name="%s" value="%s" id="%s" class="size-range" size="5" style="width: 75px">', esc_attr( $widget->get_field_name( $field ) ), esc_html( $instance[ $field ] ), esc_attr( $widget->get_field_id( $field ) ) );

						}
						?>
					</p>
						<?php
					}

					?>
				</div>
			</div>
			<div class="advanced-sidebar-menu-full-width"><!-- Clear --></div>
		</div>
		<?php
	}


	public function get_font_weights() {
		$weights = array(
			'normal'  => __( 'Normal', 'advanced-sidebar-menu' ),
			'bold'    => __( 'Bold', 'advanced-sidebar-menu' ),
			'lighter' => __( 'Light', 'advanced-sidebar-menu' ),
		);

		return apply_filters( 'advanced_sidebar_menu_font_weights', $weights );
	}


	public function get_bullet_types() {
		// @todo make this option per level
		$types = array(
			'circle'      => __( 'Circle', 'advanced-sidebar-menu' ),
			'decimal'     => __( 'Decimal', 'advanced-sidebar-menu' ),
			'disc'        => __( 'Disc', 'advanced-sidebar-menu' ),
			'square'      => __( 'Square', 'advanced-sidebar-menu' ),
			'upper-alpha' => __( 'Uppercase Alphabet', 'advanced-sidebar-menu' ),
			'lower-alpha' => __( 'Lowercase Alphabet', 'advanced-sidebar-menu' ),
			'none'        => __( 'None', 'advanced-sidebar-menu' ),
		);

		return apply_filters( 'advanced_sidebar_menu_bullet_types', $types );
	}


	public function get_border_styles( $widget_selector, array $options ) {
		$color = empty( $options[ self::BORDER_COLOR ] ) ? 'black' : $options[ self::BORDER_COLOR ];
		$out   = '';

		if ( ! empty( $options[ Advanced_Sidebar_Menu_Menus_Abstract::INCLUDE_PARENT ] ) ) {
			$out .= $widget_selector . '> ul{
			border: 1px solid ' . $color . ' !important;
			padding: 0 !important;
			}';
		} else {
			$out .= $widget_selector . '> ul{
			border-left: 1px solid ' . $color . ' !important; 
			border-right: 1px solid ' . $color . ' !important;
			border-bottom: 1px solid ' . $color . ' !important;
			padding: 0 !important;
			}';
		}
		if ( ! empty( $options[ Advanced_Sidebar_Menu_Menus_Abstract::INCLUDE_CHILDLESS_PARENT ] ) ) {
			// because we don't want to risk doubling up borders if only the top parent is shown
			$out .= $widget_selector . ' .child-sidebar-menu > li > a{border-top: 1px solid ' . $color . ' !important;}';
		} else {
			$out .= $widget_selector . ' .parent-sidebar-menu > li > a{border-bottom: 1px solid ' . $color . ' !important;}';
		}

		return $out;
	}


	public function get_block_styles( $widget_selector, $options ) {
		// make room for icons
		if ( ! empty( $options[ Advanced_Sidebar_Menu_Pro__Widget_Options__Shared__Accordion::ENABLE_ACCORDION ] ) ) {
			$out = $widget_selector . 'a{display:block !important; padding: 5px 25px 5px 10px !important;}';
		} else {
			$out = $widget_selector . 'a{display:block !important; padding: 5px 10px !important;}';
		}

		// ul ol
		$out .= $widget_selector . 'ul,' . $widget_selector . 'ol{margin: 0 0 1.75em !important; padding-left: 0 !important;}';
		$out .= $widget_selector . 'ul ul,' . $widget_selector . 'ol ol{margin: 0 0 0 !important; padding: 0 !important;}';
		$out .= $widget_selector . 'li{margin: 0 !important; padding: 0 !important}';

		// li
		$out .= $widget_selector . 'li li a{padding-left: 20px !important;}';
		$out .= $widget_selector . 'li li li a{padding-left: 30px !important;}';
		$out .= $widget_selector . 'li li li li a{padding-left: 40px !important;}';
		$out .= $widget_selector . 'li li li li li a{padding-left: 50px !important;}';

		// icons
		$out .= $widget_selector . 'li{position:relative !important}';
		$out .= $widget_selector . '.dashicons{position:absolute !important; top: 5px !important; right: 5px !important;}';

		return $out;

	}


	/**
	 * Only call when needed
	 *
	 * @return void
	 */
	public function frontend_styles() {
		wp_enqueue_script( 'asm-pro-front-end', ADVANCED_SIDEBAR_MENU_PRO_URL . 'assets/js/front-end.js', array( 'jquery' ), ADVANCED_SIDEBAR_MENU_PRO_VERSION, true );
		wp_localize_script( 'asm-pro-front-end', 'asm_pro_config', $this->js_config() );
	}


	/**
	 * Beaver build (and possible other future page builders) require scripts
	 * to be loaded on the front-end.
	 * This causes conflicts with color-pickers etc.
	 *
	 * @todo if additional use cases start creeping up, build something more
	 *       robust for handling this
	 *
	 * @return void
	 */
	public function load_scripts_on_front_end() {
		//phpcs:disable
		wp_enqueue_script( 'iris', admin_url( '/js/iris.min.js' ), array(
			'jquery-ui-draggable',
			'jquery-ui-slider',
			'jquery-touch-punch',
		), false, true );
		wp_enqueue_script( 'wp-color-picker', admin_url( '/js/color-picker.min.js' ), array( 'iris' ), false, true );
		wp_enqueue_style( 'wp-color-picker', '/css/color-picker.min.css', array(), false, true );
		wp_localize_script( 'wp-color-picker', 'wpColorPickerL10n', array(
			'clear'            => __( 'Clear' ),
			'clearAriaLabel'   => __( 'Clear color' ),
			'defaultString'    => __( 'Default' ),
			'defaultAriaLabel' => __( 'Select default color' ),
			'pick'             => __( 'Select Color' ),
			'defaultLabel'     => __( 'Color value' ),
		) );
		//phpcs:enable
		advanced_sidebar_menu_script();

		$this->widget_scripts();
	}

	/**
	 * Only call when needed
	 *
	 * @return void
	 */
	public function widget_scripts() {
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_script( 'wp-color-picker' );

		wp_enqueue_style( 'asm-widgets', ADVANCED_SIDEBAR_MENU_PRO_URL . 'assets/css/asm-widgets.css', array(), ADVANCED_SIDEBAR_MENU_PRO_VERSION );
		wp_enqueue_script( 'asm-widgets', ADVANCED_SIDEBAR_MENU_PRO_URL . 'assets/js/asm-widgets.js', array( 'underscore' ), ADVANCED_SIDEBAR_MENU_PRO_VERSION );
		wp_localize_script( 'asm-widgets', 'asm_pro_config', $this->js_config() );
	}


	private function js_config() {
		return array(
			'accordion_styles'    => $this->get_accordion_style_options(),
			'accordion_style_key' => Advanced_Sidebar_Menu_Pro__Widget_Options__Shared__Accordion::ACCORDION_STYLE,
			'i18n'                => array(
				'show_styles' => __( 'Click to show styles', 'advanced-sidebar-menu' ),
				'hide_styles' => __( 'Click to hide styles', 'advanced-sidebar-menu' ),
			),
		);
	}


	public function get_accordion_style_options() {
		$options = array(
			'plus'        => array(
				'label'    => __( 'Plus', 'advanced-sidebar-menu' ),
				'expand'   => '<span class="dashicons dashicons-plus"></span>',
				'contract' => '<span class="dashicons dashicons-minus"></span>',
			),
			'arrow'       => array(
				'label'    => __( 'Arrow', 'advanced-sidebar-menu' ),
				'expand'   => '<span class="dashicons dashicons-arrow-down-alt2"></span>',
				'contract' => '<span class="dashicons dashicons-arrow-up-alt2"></span>',
			),
			'solid_arrow' => array(
				'label'    => __( 'Solid Arrow', 'advanced-sidebar-menu' ),
				'expand'   => '<span class="dashicons dashicons-arrow-down"></span>',
				'contract' => '<span class="dashicons dashicons-arrow-up"></span>',
			),
			'hamburger'   => array(
				'label'    => __( 'Hamburger', 'advanced-sidebar-menu' ),
				'expand'   => '<span class="dashicons dashicons-menu"></span>',
				'contract' => '<span class="dashicons dashicons-minus"></span>',
			),
		);

		return apply_filters( 'advanced_sidebar_menu_accordion_styles', $options );
	}

	// ********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 */
	private static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::get_instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function get_instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}
