<?php

/**
 * Display Everywhere Option for Category Widget
 *
 * @author Mat Lipe
 * @since  3.1.0
 *
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Display_Everywhere extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {
	const NAME = 'display_everywhere';
	const PARENT_CATEGORY = 'display_everywhere/parent_category';


	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/category/is-displayed', array( $this, 'display_always' ), 10, 4 );
		add_filter( 'advanced_sidebar_menu_category_ids', array( $this, 'set_top_level_terms' ), 10, 4 );

		add_action( 'advanced-sidebar-menu/widget/category/right-column', array( $this, 'render_fields' ), 5, 2 );
	}


	/**
	 *
	 * @param bool  $displayed
	 * @param array $widget_args
	 * @param array $instance
	 *
	 * @return bool
	 */
	public function display_always( $displayed, array $widget_args, array $instance ) {
		if ( $this->checked( self::NAME, $instance ) ) {
			return true;
		}

		return $displayed;

	}


	/**
	 *
	 * @param       $term_ids
	 * @param array $widget_args
	 * @param array $instance
	 *
	 * @return array|int|WP_Error
	 */
	public function set_top_level_terms( $term_ids, array $widget_args, array $instance ) {
		if ( $this->checked( self::NAME, $instance ) ) {
			//specified one only
			if ( ! empty( $instance[ self::PARENT_CATEGORY ] ) ) {
				return array( (int) $instance[ self::PARENT_CATEGORY ] );
			}

			//all top level terms
			return get_terms( array(
				'parent'     => 0,
				'fields'     => 'ids',
				'taxonomy'   => $this->get_taxonomy( $instance ),
				'hide_empty' => true,
			) );
		}

		return $term_ids;
	}


	/**
	 *
	 * @param array                                 $instance
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::NAME ]            = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';
		$instance[ self::PARENT_CATEGORY ] = isset( $instance[ self::PARENT_CATEGORY ] ) ? $instance[ self::PARENT_CATEGORY ] : 0;

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::NAME, 'parent-category-wrap' ); ?>
				<label>
					<?php esc_html_e( 'Display widget on every page', 'advanced-sidebar-menu' ); ?>
				</label>
				<br/>
				<span class="description">
				<em><?php esc_html_e( 'Displays the same menu on every page that has this sidebar, including the homepage if applicable.', 'advanced-sidebar-menu' ); ?></em>
			</span>
			</p>
			<div <?php $widget->hide_element( self::NAME, 'parent-category-wrap' ); ?>>
				<p>
					<?php
					/* translators: Selected post type singular label */
					printf( esc_html__( 'Top level parent %s to use for this widget', 'advanced-sidebar-menu' ), esc_html( $this->get_category_label( $instance ) ) ); ?>

					<?php
					$_args = array(
						'taxonomy'        => $this->get_taxonomy( $instance ),
						'name'            => $widget->get_field_name( self::PARENT_CATEGORY ),
						'id'              => $widget->get_field_id( self::PARENT_CATEGORY ),
						'selected'        => $instance[ self::PARENT_CATEGORY ],
						'hierarchical'    => true,
						'depth'           => 1,
						/* translators: plural name of taxonomy */
						'show_option_all' => sprintf( __( '-- All %s --', 'advanced-sidebar-menu' ), $this->get_categories_label( $instance ) ),
					);

					wp_dropdown_categories( $_args ); // phpcs:ignore
					?>
				</p>
			</div>
		</div>
		<?php
	}


	protected function get_taxonomy( array $instance ) {
		return Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Taxonomy::instance()->get_taxonomy_from_option( 'category', array(), $instance );
	}


	public function get_category_label( array $instance ) {
		$taxonomy = $this->get_taxonomy( $instance );

		return get_taxonomy( $taxonomy )->labels->singular_name;
	}


	public function get_categories_label( array $instance ) {
		$taxonomy = $this->get_taxonomy( $instance );

		return get_taxonomy( $taxonomy )->labels->name;
	}


	//********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}


}
