<?php

/**
 * Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Display_Posts
 *
 * @author Mat Lipe
 * @since  3.2.0
 *
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Display_Posts extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {
	const NAME = 'display-posts';

	const POST_TYPE = 'display-posts/post-type';
	const LIMIT = 'display-posts/limit';

	protected $defaults = array(
		self::NAME      => '',
		self::POST_TYPE => 'post',
		self::LIMIT     => 20,
	);


	protected function hook() {
		add_action( 'advanced-sidebar-menu/widget/category/left-column', array( $this, 'render_fields' ), 6, 2 );

		add_filter( 'advanced-sidebar-menu/menus/category/get-list-categories-args', array(
			$this,
			'use_display_posts_walker',
		), 11, 3 );
	}


	/**
	 *
	 * @param array                                $args
	 * @param string                               $level
	 * @param Advanced_Sidebar_Menu_Menus_Category $menu
	 *
	 * @return array
	 */
	public function use_display_posts_walker( $args, $level, $menu ) {
		if ( 'parent' === $level ) {
			return $args;
		}
		if ( $this->checked( self::NAME, $menu->get_widget_instance() ) ) {
			$args['walker'] = new Advanced_Sidebar_Menu_Pro__Walkers__Display_Posts( $menu->get_widget_instance() );
		}

		return $args;
	}


	/**
	 *
	 * @param array                                  $instance
	 * @param \Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = wp_parse_args( $instance, $this->defaults );
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::NAME, 'display-posts-wrap' ); ?>
				<label>
					<?php
					/* translators: Selected {post type plural label}, Selected {taxonomy plural label} */
					printf( esc_html_x( 'Display assigned %1$s under %2$s', '{post type plural label},{taxonomy plural label}', 'advanced-sidebar-menu' ), esc_html( $this->get_post_types_label( $instance ) ), esc_html( Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Display_Everywhere::instance()->get_categories_label( $instance ) ) ); ?>
				</label>
			</p>
			<div <?php $widget->hide_element( self::NAME, 'display-posts-wrap' ); ?>>
				<?php
				$post_types = $this->get_post_types( $instance );
				unset( $post_types['post'] );
				if ( ! empty( $post_types ) ) {
					?>
					<p>
						<label>
							<?php esc_html_e( 'Post Type', 'advanced-sidebar-menu' ); ?>:
						</label>
						<select id="<?php echo esc_attr( $widget->get_field_id( self::POST_TYPE ) ); ?>"
						        name="<?php echo esc_attr( $widget->get_field_name( self::POST_TYPE ) ); ?>">
							<option value="post">
								<?php esc_html_e( 'Posts', 'advanced-sidebar-menu' ); ?>
							</option>
							<?php
							foreach ( $post_types as $_name => $_label ) {
								printf( '<option value="%s" %s">%s</option>', esc_attr( $_name ), selected( $_name, $instance[ self::POST_TYPE ] ), esc_html( $_label ) );

							}
							?>
						</select>
					</p>
					<p>
						<label>
							<?php
							/* translators: Selected post type plural label */
							printf( esc_html__( 'Number of %s to display', 'advanced-sidebar-menu' ), esc_html( $this->get_post_types_label( $instance ) ) ); ?>
							:
						</label>
						<select
							id="<?php echo esc_attr( $widget->get_field_id( self::LIMIT ) ); ?>"
							name="<?php echo esc_attr( $widget->get_field_name( self::LIMIT ) ); ?>">
							<option value="100">
								<?php esc_html_e( '-- All --', 'advanced-sidebar-menu' ); ?>
							</option>
							<?php
							for ( $i = 1; $i <= 25; $i ++ ) {
								printf( '<option value="%s" %s">%s</option>', (int) $i, selected( $i, $instance[ self::LIMIT ] ), (int) $i );
							}
							?>
						</select>
					</p>
					<?php
				}
				?>
			</div>
		</div>
		<?php

	}


	public function get_post_types_label( $instance ) {
		if ( empty( $instance[ self::POST_TYPE ] ) ) {
			return 'posts';
		}
		$pt = get_post_type_object( $instance[ self::POST_TYPE ] );
		if ( null === $pt ) {
			return 'posts';
		}

		return $pt->labels->name;
	}


	/**
	 * Get available post types for this widget
	 *
	 *
	 * @param array $instance
	 *
	 * @access public for unit testing
	 *
	 * @return array
	 */
	public function get_post_types( array $instance ) {
		$tax = get_taxonomy( Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Taxonomy::instance()->get_taxonomy_from_option( 'category', array(), $instance ) );

		$post_types = array();
		$objects    = get_post_types( array(), array( 'output' => 'objects' ) );
		foreach ( $objects as $_object ) {
			if ( in_array( $tax->name, $_object->taxonomies, true ) || in_array( $_object->name, $tax->object_type, true ) ) {
				$post_types[ $_object->name ] = $_object->labels->name;
			}
		}

		$post_types = apply_filters( 'advanced-sidebar-menu/widget/category/available-post-types', $post_types, $instance );

		return $post_types;
	}


	//********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}


}
