<?php


/**
 * Option to order categories by a few fields
 *
 * @since 1.1.0
 *
 * @uses  order the category results by a choice
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Order_By extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {

	const NAME = true;

	const ORDER = 'order';
	const ORDER_BY = 'order_by';

	//order by
	const ORDER_BY_NAME = 'name';
	const ORDER_BY_COUNT = 'count';
	const ORDER_BY_SLUG = 'slug';
	//order
	const ORDER_ASC = 'ASC';
	const ORDER_DESC = 'DESC';


	protected function hook() {
		add_filter( 'advanced_sidebar_menu_category_orderby', array( $this, 'filter_orderby' ), 10, 3 );
		add_filter( 'advanced_sidebar_menu_category_order', array( $this, 'filter_order' ), 10, 3 );

		add_action( 'advanced-sidebar-menu/widget/category/right-column', array( $this, 'render_fields' ), 15, 2 );
	}


	/**
	 *
	 * @param array      $instance
	 * @param \Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance[ self::ORDER ]    = empty( $instance[ self::ORDER ] ) ? 'ASC' : $instance[ self::ORDER ];
		$instance[ self::ORDER_BY ] = empty( $instance[ self::ORDER_BY ] ) ? '' : $instance[ self::ORDER_BY ];
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label>
					<?php esc_html_e( 'Order By', 'advanced-sidebar-menu' ); ?>:
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::ORDER_BY ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::ORDER_BY ) ); ?>">
					<option value="">
						<?php esc_html_e( 'Created', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					$order_by = array(
						self::ORDER_BY_NAME  => __( 'Name', 'advanced-sidebar-menu' ),
						self::ORDER_BY_COUNT => __( 'Count', 'advanced-sidebar-menu' ),
						self::ORDER_BY_SLUG  => __( 'Slug', 'advanced-sidebar-menu' ),
					);

					foreach ( $order_by as $key => $order ) {
						printf( '<option value="%s" %s>%s</option>', esc_attr( $key ), selected( $instance[ self::ORDER_BY ], $key, false ), esc_html( $order ) );
					}
					?>
				</select>
			</p>
			<p> <?php esc_html_e( 'Order', 'advanced-sidebar-menu' ); ?>:
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::ORDER ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::ORDER ) ); ?>">
					<?php
					$order = array(
						self::ORDER_ASC  => __( 'Ascending', 'advanced-sidebar-menu' ),
						self::ORDER_DESC => __( 'Descending', 'advanced-sidebar-menu' ),
					);

					foreach ( $order as $key => $_order ) {

						printf( '<option value="%s" %s>%s</option>', esc_attr( $key ), selected( $instance[ self::ORDER ], $key, false ), esc_html( $_order ) );
					}
					?>
				</select>
			</p>
		</div>
		<?php
	}


	public function filter_orderby( $orderby, $args, $instance ) {
		if ( isset( $instance[ self::ORDER_BY ] ) ) {
			return $instance[ self::ORDER_BY ];
		}

		return $orderby;
	}


	public function filter_order( $order, $args, $instance ) {
		if ( isset( $instance[ self::ORDER ] ) ) {
			return $instance[ self::ORDER ];
		}

		return $order;
	}


	//********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}
