<?php


/**
 * Advanced_Sidebar_Menu_Pro__Widget_Options__Page__Limit_Levels
 *
 * Limit the levels of pages in the output
 *
 * @author Mat Lipe
 *
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Page__Limit_Levels extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {

	const NAME = 'level_limit';


	protected function hook() {
		add_filter( 'advanced-sidebar-menu/menus/page/depth', array( $this, 'limit_page_levels' ), 10, 3 );
		add_action( 'advanced-sidebar-menu/widget/page/display-box', array( $this, 'render_fields' ), 15, 2 );
	}


	/**
	 * If we have a limit set we limit the levels via filter
	 *
	 * @param int   $levels
	 * @param array $widget_args
	 * @param array $instance
	 *
	 * @return mixed
	 */
	public function limit_page_levels( $levels, $widget_args, array $instance ) {
		if ( ! empty( $instance[ self::NAME ] ) ) {
			//subtract 1 level to account for parent menu
			return ( $instance[ self::NAME ] - 1 );
		}

		return $levels;

	}


	/**
	 *
	 * @param array                                 $instance
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		?>
		<div <?php $widget->hide_element( Advanced_Sidebar_Menu_Widget_Page::DISPLAY_ALL,  Advanced_Sidebar_Menu_Widget_Page::LEVELS, true ); ?>>
			<p>
				<label><?php esc_html_e( 'Number of levels of pages to show', 'advanced-sidebar-menu' ); ?>:
				</label>
				<?php
				if ( empty( $instance[ self::NAME ] ) ) {
					$instance[ self::NAME ] = 100;
				}
				?>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::NAME ) ); ?>">
					<option value="100">
						<?php esc_html_e( ' - All - ', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					for ( $i = 1; $i < 10; $i ++ ) {
						printf( '<option value="%s" %s">%s</option>', esc_attr( $i ), selected( $i, $instance[ self::NAME ] ), esc_attr( $i ) );
					}
					?>
				</select>
			</p>
		</div>
		<?php
	}


	//********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}
