<?php


/**
 * Advanced_Sidebar_Menu_Pro__Widget_Options__Page__Post_Type
 *
 * @author Mat Lipe
 * @since  2.3.0
 *
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Page__Post_Type extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {

	const NAME = 'post_type';


	protected function hook() {
		add_filter( 'advanced_sidebar_menu_post_type', array( $this, 'get_post_type_from_option' ), 10, 3 );

		add_action( 'advanced-sidebar-menu/widget/page/right-column', array( $this, 'render_fields' ), 10, 2 );

	}


	/**
	 *
	 * @param string $post_type
	 * @param array  $widget_args
	 * @param array  $instance
	 *
	 * @return mixed
	 */
	public function get_post_type_from_option( $post_type, array $widget_args, array $instance ) {
		if ( ! empty( $instance[ self::NAME ] ) ) {
			return $instance[ self::NAME ];
		}

		return $post_type;
	}


	/**
	 *
	 * @param array                                 $instance
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$post_types = $this->get_post_types();
		if ( empty( $post_types ) ) {
			return;
		}
		$instance[ self::NAME ] = isset( $instance[ self::NAME ] ) ? $instance[ self::NAME ] : '';
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label>
					<?php esc_html_e( 'Post Type', 'advanced-sidebar-menu' ); ?>:
				</label>
				<select id="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>"
				        name="<?php echo esc_attr( $widget->get_field_name( self::NAME ) ); ?>">
					<option value="0">
						<?php esc_html_e( 'Pages', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					foreach ( $post_types as $_name => $_label ) {
						printf( '<option value="%s" %s">%s</option>', esc_attr( $_name ), selected( $_name, $instance[ self::NAME ] ), esc_html( $_label ) );

					}
					?>
				</select>
			</p>
		</div>
		<?php
	}


	/**
	 * Get available post types for this widget
	 *
	 * @access public for unit testing
	 *
	 * @return array
	 */
	public function get_post_types() {
		$post_types = array();
		$objects    = get_post_types( array(
			'hierarchical' => true,
			'_builtin'     => false,
		), array( 'output' => 'objects' ) );
		foreach ( $objects as $_object ) {
			$post_types[ $_object->name ] = $_object->labels->name;
		}

		$post_types = apply_filters( 'advanced_sidebar_menu_available_post_types', $post_types );

		return $post_types;
	}

	//********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}
