<?php

/**
 * Advanced_Sidebar_Menu_Pro__Widget_Options__Shared__Accordion
 *
 * @author Mat Lipe
 * @since  1.5
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Shared__Accordion extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {
	const NAME = 'enable_accordion';

	const ENABLE_ACCORDION = self::NAME;
	const ACCORDION_STYLE = 'accordion_style';
	const ICON_COLOR = 'icon_color';
	const KEEP_CLOSED = 'keep_closed';
	const INCLUDE_PARENT = 'include_parent_in_accordion';

	/**
	 * Default settings for this option.
	 *
	 * @var array
	 */
	private $defaults = array(
		self::ENABLE_ACCORDION => 0,
		self::ACCORDION_STYLE  => 'plus',
		self::ICON_COLOR       => false,
		self::KEEP_CLOSED      => false,
		self::INCLUDE_PARENT   => false,
	);


	/**
	 * Add actions.
	 *
	 * @return void
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/menus/page/render', array( $this, 'render' ) );
		add_action( 'advanced-sidebar-menu/menus/category/render', array( $this, 'render' ) );
		add_action( 'advanced-sidebar-menu/menus/navigation/render', array( $this, 'render' ) );

		add_action( 'advanced-sidebar-menu/widget/page/right-column', array( $this, 'render_fields' ), 20, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/right-column', array( $this, 'render_fields' ), 20, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', array( $this, 'render_fields' ), 10, 2 );
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array                                 $instance - widget settings.
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget - widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = wp_parse_args( $instance, $this->defaults );

		$style_options = Advanced_Sidebar_Menu_Pro_Styles::get_instance()->get_accordion_style_options();

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::ENABLE_ACCORDION, 'accordion-wrap' ); ?>
				<label>
					<?php esc_html_e( 'Enable accordion menu', 'advanced-sidebar-menu' ); ?>
				</label>
				<br/>
				<span class="description">
				<em><?php esc_html_e( 'It is recommended that you use the "Always display child pages" option when using this feature.', 'advanced-sidebar-menu' ); ?></em>
			</span>
			</p>
			<div <?php $widget->hide_element( self::ENABLE_ACCORDION, 'accordion-wrap' ); ?>>
				<p>
					<label>
						<?php esc_html_e( 'Accordion icons style', 'advanced-sidebar-menu' ); ?>:
					</label>
					<select
						data-js="advanced-sidebar-menu-pro/widget/accordion-style"
						id="<?php echo esc_attr( $widget->get_field_id( self::ACCORDION_STYLE ) ); ?>"
						name="<?php echo esc_attr( $widget->get_field_name( self::ACCORDION_STYLE ) ); ?>"
						title="<?php esc_attr_e( 'Accordion Style', 'advanced-sidebar-menu' ); ?>">
						<?php
						foreach ( $style_options as $_style => $_icons ) {
							printf( '<option value="%s" %s>%s</option>', esc_attr( $_style ), selected( $_style, $instance[ self::ACCORDION_STYLE ] ), esc_html( $_icons['label'] ) );
						}
						?>
					</select>
					<br/>
					<span data-select="<?php echo esc_attr( $widget->get_field_id( self::ACCORDION_STYLE ) ); ?>">
						<?php
						echo wp_kses( $style_options[ $instance[ self::ACCORDION_STYLE ] ]['expand'] . ' ' . $style_options[ $instance[ self::ACCORDION_STYLE ] ]['contract'], array( 'span' => array( 'class' => array() ) ) );
						?>
					</span>
				</p>
				<p>
					<label class="asm-css-widget-label">
						<?php esc_html_e( 'Accordion icons color', 'advanced-sidebar-menu' ); ?>:
					</label>
					<?php
					printf( '<input type="text" name="%s" value="%s" id="%s" class="asm-color-select">', esc_attr( $widget->get_field_name( self::ICON_COLOR ) ), esc_attr( $instance[ self::ICON_COLOR ] ), esc_attr( $widget->get_field_id( self::ICON_COLOR ) ) );
					?>
				</p>
				<p>
					<?php $widget->checkbox( self::KEEP_CLOSED ); ?>

					<label>
						<?php esc_html_e( 'Keep all accordion sections closed until clicked', 'advanced-sidebar-menu' ); ?>
					</label>
					<br/>
					<span class="description">
						<em><?php esc_html_e( 'If this option is checked, all accordion items will remain closed until clicked. If unchecked, the menu will open the menu to the current section automatically.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
				<p>
					<?php $widget->checkbox( self::INCLUDE_PARENT ); ?>
					<label>
						<?php esc_html_e( 'Include highest level parent in accordion', 'advanced-sidebar-menu' ); ?>
					</label>
					<br/>
					<span class="description">
				<em><?php esc_html_e( 'If this option is checked the highest level parent will become part of the accordion open/close ability.', 'advanced-sidebar-menu' ); ?></em>
			</span>
				</p>
			</div>
		</div>
		<?php
	}


	/**
	 * Render the required js and css on front end
	 *
	 * @param Advanced_Sidebar_Menu_Menus_Abstract $menu - menu class.
	 *
	 * @return void
	 */
	public function render( $menu ) {
		$instance    = $menu->get_widget_instance();
		$widget_args = $menu->get_widget_args();
		if ( empty( $instance[ self::NAME ] ) ) {
			return;
		}

		wp_enqueue_style( 'dashicons' );
		Advanced_Sidebar_Menu_Pro_Styles::get_instance()->frontend_styles();

		if ( ! empty( $instance[ self::ICON_COLOR ] ) ) {
			?>
			<style>
				#<?php echo esc_attr( $widget_args['widget_id'] ); ?>-wrap .dashicons {
					color: <?php echo esc_attr( $instance[ self::ICON_COLOR ] ); ?> !important;
					cursor: pointer;
				}
			</style>
			<?php
		}

		// Register the accordion with the front-end JS. Will output in the footer of the site.
		wp_add_inline_script( 'asm-pro-front-end', '
			jQuery( document ).on( "advanced-sidebar-menu-pro/accordions/available", function ( ev, accordion ) {
				accordion.register_accordion("' . esc_js( $widget_args['widget_id'] ) . '-wrap",' . wp_json_encode( $instance ) . ' );
			});',
			'before' );
	}


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}
