<?php

/**
 * Advanced_Sidebar_Menu_Pro__Widget_Options__Shared__Shown_Styles
 *
 * @author Mat Lipe
 * @since  3.2.0
 *
 */
class Advanced_Sidebar_Menu_Pro__Widget_Options__Shared__Shown_Styles extends Advanced_Sidebar_Menu_Pro__Widget_Options__Fields {

	const APPLY_CURRENT_STYLES = 'apply_current_page_styles_to_parent';
	const APPLY_PARENT_STYLES = 'apply_current_page_parent_styles_to_parent';

	private $defaults = array(
		Advanced_Sidebar_Menu_Pro_Styles::BLOCK_STYLE  => false,
		Advanced_Sidebar_Menu_Pro_Styles::BORDER       => false,
		Advanced_Sidebar_Menu_Pro_Styles::BULLET_STYLE => false,
		Advanced_Sidebar_Menu_Pro_Styles::BORDER_COLOR => false,
	);


	protected function hook() {
		add_action( 'advanced-sidebar-menu/widget/page/styles-box', array( $this, 'render_fields' ), 10, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/styles-box', array( $this, 'render_fields' ), 10, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/styles-box', array( $this, 'render_fields' ), 10, 2 );
	}


	/**
	 *
	 * @param array                                 $instance
	 * @param Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @notice Front end rendering is handled by the Styles class
	 * @see    Advanced_Sidebar_Menu_Pro__Widget_Options__Page__Styles::render()
	 * @see    Advanced_Sidebar_Menu_Pro__Widget_Options__Category__Styles::render()
	 *
	 * @todo   Migrate keys between the two widgets so we can simply have different labels
	 *       but store same keys in db
	 *
	 * @return mixed|void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = wp_parse_args( $instance, $this->defaults );

		?>
		<p>
			<?php $widget->checkbox( self::APPLY_CURRENT_STYLES ); ?>
			<label>
				<?php esc_html_e( 'Apply current item styles to highest level parent page (when applicable)', 'advanced-sidebar-menu' ); ?>
			</label>

		</p>
		<p>
			<?php $widget->checkbox( self::APPLY_PARENT_STYLES ); ?>
			<label>
				<?php esc_html_e( 'Apply current item parent styles to highest level parent page (when applicable)', 'advanced-sidebar-menu' ); ?>
			</label>
		</p>

		<p>
			<?php $widget->checkbox( Advanced_Sidebar_Menu_Pro_Styles::BLOCK_STYLE, 'border-wrap' ); ?>
			<label><?php esc_html_e( 'Use block styling', 'advanced-sidebar-menu' ); ?></label>
		</p>
		<div <?php $widget->hide_element( Advanced_Sidebar_Menu_Pro_Styles::BLOCK_STYLE, 'border-wrap' ); ?>>
			<p>
				<?php $widget->checkbox( Advanced_Sidebar_Menu_Pro_Styles::BORDER, 'border-color-wrap' ); ?>
				<label><?php esc_html_e( 'Show borders', 'advanced-sidebar-menu' ); ?></label>
			</p>

			<div <?php $widget->hide_element( Advanced_Sidebar_Menu_Pro_Styles::BORDER, 'border-color-wrap' ); ?>>
				<p>
					<label>
						<?php esc_html_e( 'Border color', 'advanced-sidebar-menu' ); ?>:
					</label>
					<?php
					printf( '<input type="text" name="%s" value="%s" id="%s" class="asm-color-select">',
						esc_attr( $widget->get_field_name( Advanced_Sidebar_Menu_Pro_Styles::BORDER_COLOR ) ),
						esc_html( $instance[ Advanced_Sidebar_Menu_Pro_Styles::BORDER_COLOR ] ),
						esc_attr( $widget->get_field_id( Advanced_Sidebar_Menu_Pro_Styles::BORDER_COLOR ) ) );
					?>
				</p>
			</div>
		</div>
		<p>
			<label>
				<?php esc_html_e( 'Bullet style', 'advanced-sidebar-menu' ); ?>:
			</label>
			<select
				name="<?php echo esc_attr( $widget->get_field_name( Advanced_Sidebar_Menu_Pro_Styles::BULLET_STYLE ) ); ?>"
				id="<?php echo esc_attr( $widget->get_field_id( Advanced_Sidebar_Menu_Pro_Styles::BULLET_STYLE ) ); ?>">
				<option value="">
					<?php esc_html_e( '- default -', 'advanced-sidebar-menu' ); ?>
				</option>
				<?php
				foreach ( (array) Advanced_Sidebar_Menu_Pro_Styles::get_instance()->get_bullet_types() as $_type => $__label ) {
					printf( '<option value="%s" %s>%s</option>', esc_attr( $_type ), selected( $instance[ Advanced_Sidebar_Menu_Pro_Styles::BULLET_STYLE ], esc_attr( $_type ) ), esc_html( $__label ) );
				}
				?>
			</select>
		</p>
		<?php
	}


	//********** SINGLETON **********/


	/**
	 * Instance of this class for use as singleton
	 *
	 * @var self
	 */
	protected static $instance;


	/**
	 * Create the instance of the class
	 *
	 * @static
	 * @return void
	 */
	public static function init() {
		self::instance()->hook();
	}


	/**
	 * Get (and instantiate, if necessary) the instance of the
	 * class
	 *
	 * @static
	 * @return self
	 */
	public static function instance() {
		if ( ! is_a( self::$instance, __CLASS__ ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}
