<?php

/**
 * Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu
 *
 * @author Mat Lipe
 * @since  3.3.0
 *
 */
class Advanced_Sidebar_Menu_Pro__Widgets__Navigation_Menu extends Advanced_Sidebar_Menu__Widget__Widget {
	const TITLE = Advanced_Sidebar_Menu_Menus_Abstract::TITLE;
	const NAVIGATION_MENU = 'navigation_menu';
	const USE_PLUGIN_STYLES = Advanced_Sidebar_Menu_Menus_Abstract::USE_PLUGIN_STYLES;
	const CURRENT_PARENT_ONLY = 'current-parent-only';
	const TOP_LEVEL_SIBLINGS = 'top-level-siblings';
	const DISPLAY_NO_CURRENT = 'display-no-current';


	protected static $defaults = array(
		self::TITLE               => false,
		self::NAVIGATION_MENU     => false,
		self::USE_PLUGIN_STYLES   => false,
		self::CURRENT_PARENT_ONLY => false,
		self::TOP_LEVEL_SIBLINGS  => false,
		self::DISPLAY_NO_CURRENT  => false,
	);

	protected static $hooked = false;


	public function __construct() {
		$widget_ops  = array(
			'classname'   => 'advanced-sidebar-menu advanced-sidebar-navigation',
			'description' => __( 'Creates a sidebar menu from a Navigation Menu in Appearance -> Menus', 'advanced-sidebar-menu' ),
		);
		$control_ops = array(
			'width' => 620,
		);

		parent::__construct( 'advanced-sidebar-navigation-menu', __( 'Advanced Sidebar Navigation Menu', 'advanced-sidebar-menu' ), $widget_ops, $control_ops );

		if ( ! self::$hooked ) {
			self::$hooked = true;
			$this->hook();
		}

	}


	/**
	 * @notice Anything using the column actions must use the $widget class passed
	 *         via do_action instead of $this
	 *
	 * @return void
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/left-column', array( $this, 'box_menu' ), 5, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/left-column', array( $this, 'box_styles' ), 10, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', array(
			$this,
			'box_display_parent_only',
		), 10, 2 );


	}


	/**
	 * Update
	 *
	 * @param array $new_instance
	 * @param array $old_instance
	 *
	 * @return array|mixed
	 */
	public function update( $new_instance, $old_instance ) {
		return apply_filters( 'advanced-sidebar-menu/widget/navigation-menu/update', $new_instance, $old_instance );
	}


	/**
	 *
	 * @param array                                  $instance
	 * @param \Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function box_menu( array $instance, $widget ) {
		$menus    = wp_get_nav_menus();
		$instance = $widget->set_instance( $instance, self::$defaults );
		$nav_menu = isset( $instance[ self::NAVIGATION_MENU ] ) ? $instance[ self::NAVIGATION_MENU ] : '';

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label>
					<?php esc_html_e( 'Select Menu:', 'advanced-sidebar-menu' ); ?>
				</label>
				<select id="<?php echo esc_attr( $widget->get_field_id( self::NAVIGATION_MENU ) ); ?>"
				        name="<?php echo esc_attr( $widget->get_field_name( self::NAVIGATION_MENU ) ); ?>">
					<option value="0">
						<?php esc_html_e( '&mdash; Select &mdash;', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					foreach ( $menus as $menu ) {
						?>
						<option
							value="<?php echo esc_attr( $menu->term_id ); ?>" <?php selected( $nav_menu, $menu->term_id ); ?>>
							<?php echo esc_html( $menu->name ); ?>
						</option>
						<?php
					}
					?>
				</select>
			</p>
		</div>
		<?php

	}

	/**
	 *
	 * @param array                                  $instance
	 * @param \Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function box_styles( array $instance, $widget ) {
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::USE_PLUGIN_STYLES ); ?>
				<label>
					<?php esc_html_e( "Use this plugin's default styling", 'advanced-sidebar-menu' ); ?>
				</label>
			</p>
			<?php do_action( 'advanced-sidebar-menu/widget/navigation-menu/styles-box', $instance, $widget ); ?>
		</div>
		<?php
	}

	/**
	 *
	 * @param array                                  $instance
	 * @param \Advanced_Sidebar_Menu__Widget__Widget $widget
	 *
	 * @return void
	 */
	public function box_display_parent_only( array $instance, $widget ) {
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::CURRENT_PARENT_ONLY, 'siblings-no-children' ); ?>
				<label><?php esc_html_e( "Display the current item's parents only.", 'advanced-sidebar-menu' ); ?>
				</label>
				<br/>
				<span class="description">
					<em>
						<?php esc_html_e( 'Only display the current item and any parents, grandparents, or children of the current item.', 'advanced-sidebar-menu' ); ?>
					</em>
				</span>
			</p>
			<div <?php $widget->hide_element( self::CURRENT_PARENT_ONLY, 'siblings-no-children' ); ?>>

				<p>
					<?php $widget->checkbox( self::TOP_LEVEL_SIBLINGS, 'siblings-wrap' ); ?>
					<label>
						<?php esc_html_e( 'Display top level siblings.', 'advanced-sidebar-menu' ); ?>
					</label>
					<br/>
					<span class="description">
				<em><?php esc_html_e( 'Display top level menu items when on a top level menu item that has no children.', 'advanced-sidebar-menu' ); ?></em>
			</span>
				</p>

				<p>
					<?php $widget->checkbox( self::DISPLAY_NO_CURRENT ); ?>
					<label>
						<?php esc_html_e( 'Display top level items when not viewing a menu item', 'advanced-sidebar-menu' ); ?>
					</label>
					<br/>
					<span class="description">
				<em><?php esc_html_e( 'Display the top level menu items when viewing a page that is not part of the menu.', 'advanced-sidebar-menu' ); ?></em>
			</span>
				</p>
			</div>
		</div>
		<?php
	}


	/**
	 * Form
	 *
	 * @param array $instance
	 *
	 * @return void
	 */
	public function form( $instance ) {
		$instance = $this->set_instance( $instance, self::$defaults );
		do_action( 'advanced-sidebar-menu/widget/navigation-menu/before-form', $instance, $this );
		?>
		<p xmlns="http://www.w3.org/1999/html">
			<label>
				<?php esc_html_e( 'Title', 'advanced-sidebar-menu' ); ?>:
			</label>

			<input
				id="<?php echo esc_attr( $this->get_field_id( self::TITLE ) ); ?>"
				name="<?php echo esc_attr( $this->get_field_name( self::TITLE ) ); ?>"
				class="widefat"
				type="text"
				value="<?php echo esc_attr( $instance[ self::TITLE ] ); ?>"/>
		</p>
		<div class="advanced-sidebar-menu-column">
			<?php
			do_action( 'advanced-sidebar-menu/widget/navigation-menu/left-column', $instance, $this );
			?>
		</div>
		<div class="advanced-sidebar-menu-column advanced-sidebar-menu-column-right">
			<?php
			do_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', $instance, $this );
			?>
		</div>
		<div class="advanced-sidebar-menu-full-width"><!-- clear --></div>
		<?php
		do_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', $instance, $this );
	}


	public function widget( $args, $instance ) {
		$instance = wp_parse_args( $instance, self::$defaults );
		$asm      = Advanced_Sidebar_Menu_Pro__Menus__Navigation::factory( $instance, $args );

		do_action( 'advanced_sidebar_menu_widget_pre_render', $asm, $this );

		$asm->render();

	}

}
